use strict ;
our ($MW, $Balloon, @Db) ;
our (%Close) ;

sub operating_stations { 
    my @db = dblookup(@Db, 0, "site", 0, 0) ; 
    my $nulloffdate = nullvalue("offdate") ; 
    my $n = dbquery(@db, dbRECORD_COUNT) ; 
    my ($staname, %staname) ; 
    my ($sta, $staname, $offdate ) ;
    for ( $db[3] = 0 ; $db[3] < $n ; $db[3]++ ) { 
	($sta, $staname, $offdate ) =  dbgetv ( @db, qw(sta staname offdate)) ; 
	if ( $offdate == $nulloffdate ) {  
	    $staname{$sta} = $staname ; 
	}
    }
    return \%staname ; 
}

sub no_stations {
    my ($ref) = @_ ; 
    foreach (keys %$ref) { 
	$$ref{$_} = 0 ; 
    }
}

sub all_stations {
    my ($ref) = @_ ; 
    foreach (keys %$ref) { 
	$$ref{$_} = 1 ; 
    }
}

sub do_stations {
    my ($top, $stations) = @_ ; 
    my $when = $Close{time} ;
    my ($time,$msg) = is_epoch_string($when) ;
    if ( ! defined $when ) { 
	elog_complain ("must specify time station was shut down" ) ; 
    } elsif ( ! defined $time ) { 
	elog_complain ("invalid time '$when': $msg" ) ; 
    } else { 
	foreach (keys %$stations) { 
	    if ( $$stations{$_} ) { 
		close_station ($_, $time ) ; 
	    }
	    $$stations{$_} = 0 ; 
	}
	$top->destroy() ;
    }
}

sub close_close { 
    my ($ref) = @_ ; 
    $$ref->destroy ; 
}

sub pick_stations2close { 
    my $top = $MW->Toplevel(-title=>"Close Stations") ; 
    # my $f = $top->Scrolled("Frame", -scrollbars=>"oe")
    #	    ->pack(-side=>"top", -fill=>"both", -expand=>"yes");

    my %close ;

    my $f = $top->Frame()
	->pack(-side=>"top", -fill=>"both", -expand=>"yes");
    my $s = $f->Frame()
	->pack(-side=>"left", -fill=>"both", -expand=>"yes");
    $s->Label(-text=>"Shutdown time", -font=>"ptkformLabel", -fg=>"darkblue")
            ->pack(-side=>"top", -ipady=>0, -pady=>0, -anchor=>"sw" ) ;
    $s->Entry(-textvariable=>\$Close{time}, -font=>"ptkformEntry")
        ->pack(-side=>"top", -ipady=>0, -pady=>0, -anchor=>"nw" ) ;
    $f->Button(-text=>"Close Selected Stations", -font=>"ptkformLabel", -fg=>"darkblue", -command=>[\&do_stations, $top, \%close]) 
	->pack(-side=>"top", -fill=>"x", -expand=>"yes");


    my $ref = &operating_stations() ;
    my @sta = sort keys %$ref ;
    my $row = 0 ; 
    my $col = 0 ; 
    my $n = @sta ; 
    my $maxcol = 10 ; 
    if ( sqrt($n) < $maxcol ) { 
	$maxcol = sqrt($n) ; 
    }
    $f = $top->Frame() 
    	    ->pack(-side=>"top", -fill=>"both", -expand=>"yes");
    my $w ;
    foreach my $sta ( @sta ) { 
	$w = $f->Checkbutton(-text=>$sta, 
	    -indicatoron => 0,
	    -font=>"ptkformLabel", 
	    -fg=>"darkblue",
	    -variable=>\$close{$sta}) 
	->grid(-sticky=>"ew", -row=>$row, -column=>$col) ; 
	$col++ ;
	if ( $col >= $maxcol ) { 
	    $col = 0 ; 
	    $row++ ; 
	}
	$Balloon->attach($w, -balloonmsg=>$$ref{$sta}, -initwait=>2) ; 
    }

    $f = $top->Frame()
	->pack(-side=>"top", -fill=>"both", -expand=>"yes");
    $f->Button(-text=>"All", -font=>"ptkformLabel", -fg=>"darkblue", -command=>[\&all_stations, \%close])  
	->pack(-side=>"left", -fill=>"x", -expand=>"yes");
    $f->Button(-text=>"None", -font=>"ptkformLabel", -fg=>"darkblue", -command=>[\&no_stations, \%close])  
	->pack(-side=>"left", -fill=>"x", -expand=>"yes");

    $f->Button(-text=>"Cancel", -font=>"ptkformLabel", -command=>[\&close_close, \$top]) 
	->pack(-side=>"left", -fill=>"x", -expand=>"yes");

    $top->resizable("no", "no") ;
    $top->update() ;
}
    
1;
