package rt ; 

require Exporter;   
@ISA = ('Exporter');

@EXPORT=qw(rtchildren rtalarm rtbkg rtkill rtrun rtcmdout) ;

use children ;

use POSIX ":sys_wait_h";
use POSIX qw(setsid setpgid) ;

sub rtrun {
    my ( $cmd, $timeout ) = @_ ;
    my $pid ; 
    my $aborted = 0 ;

    local $SIG{'ALRM'} = sub { 
        rtkill (1, $pid) ;
	$aborted = 1 ;
    };

    alarm $timeout ;       
    $pid = fork ;
    if ( $pid ) {
	# parent
    } elsif ( defined $pid ) {
	# child
	open ( STDIN, "</dev/null" ) ;
	exec $cmd ;
        die ( "Can't exec '$cmd' : $!\n" ) ; 
    } else {
	# fork failure
        die "Can't fork '$cmd'\n" ; 
    }

    waitpid ( $pid, 0 ) ; 
    my $left  = alarm 0 ;
    return $aborted ;
}

sub rtbkg { 
    my ( $cmd ) = @_ ; 
    my ( $pid ) ;
    if ( $pid = fork ) { 
	# parent
	return  $pid ; 
    } elsif ( defined $pid ) {
	# child
	setsid() ;
        exec $cmd ; 
        die ( "Can't exec '$cmd' : $!\n" ) ; 
    } else {
	# fork failure
        die "Can't fork '$cmd'\n" ; 
    }
}

sub rtalarm { 
    my ( $pid, $timeout, $ref ) = @_ ;
    our $RtAlarm = $ref ; 
    our $RtPid = $pid ; 
    our $RtTimeout = $timeout ;
    $$RtAlarm = 0 ; 

    $SIG{'ALRM'} = sub { 
	print STDERR "Timeout after $RtTimeout seconds: Killing $RtPid\n" ; 
	$$RtAlarm = 1 ; 
        rtkill (1, $pid) ;
    };

    alarm $timeout ;       
}

sub rtkill {
    my ($time2die, @pids) = @_ ;
    my ($pid) ;
    my (@all) = (); 
    my (@all2) = () ;
    my (@subs, $i, @children) ;
    my (@msgs) = () ; 
    return if (@pids == 0 ) ;
    foreach $pid ( @pids ) {
#  suppose rtkill is called with undef: "rtkill (undef);"
	if ( defined $pid && -e "/proc/$pid/status" ) {
	    @children = &rtchildren($pid) ;
	    if ( @children > 0 ) {
		push(@msgs, "killing pid $pid and its children @children") ;
	    } else { 
		push(@msgs, "killing pid $pid") ;
	    }
	    push ( @all, $pid, @children ) ;
	    kill 'SIGTERM', $pid, @children ;
	}
    }

    for ( $i=0 ; @all > 0 && $i<$time2die ; $i++ ) {
	sleep 1 ;
	foreach $pid ( @all ) {
	    waitpid ( $pid, &WNOHANG ) ; 
	}
	@all2 = () ;
	foreach $pid ( @all ) {
	    if ( -e "/proc/$pid/status" ) {
		push (@all2, $pid ) ; 
	    }
	}
	@all = @all2 ; 
	if ( $i > 1 && $i % 1 == 0 ) { 
	    if ( @children > 0 ) {
		push(@msgs, "waiting for pids @all" ) ; 
	    } else { 
		push(@msgs, "waiting for pids @all (pid and its children)" ) ; 
	    }
	}
    }

    if ( @all > 0 ) { 
	foreach $pid ( @all ) {
	    push(@msgs, "forced to kill -9 $pid") ;
	    kill '9', $pid ;
	}
	sleep 1 ;
	foreach $pid ( @all ) {
	    waitpid ( $pid, &WNOHANG ) ; 
	}

	foreach $pid ( @all ) {
	    if ( -e "/proc/$pid/status" ) {
		push(@msgs, "can't kill $pid" ) ;
	    }
	}
    }
    return @msgs ;
}

sub rtcmdout {
    my ( $cmd, $timeout ) = @_ ;
    my $pid ; 
    my $aborted = 0 ;

    if ( defined $timeout ) { 
	local $SIG{'ALRM'} = sub { 
	    rtkill (1, $pid) ;
	    $aborted = 1 ;
	};

	alarm $timeout ;       
    }

    my @args = split(' ', $cmd) ;
    my $output = "" ;
    if (open(CMD, "-|")) {
        while (<CMD>) {
            $output .= $_;
        }
        close CMD;
    } else {
        open (STDERR, ">&STDOUT") ;
        exec { $args[0] } @args ;
        print STDERR "$!\n" ;  
        exit 1 ;
    }
    my $result = $? ;
    waitpid ( $pid, 0 ) ; 
    my $left  = alarm 0 if defined $timeout ;
    return ($result, $output) ;
}

1;
