package rtmail ; 

require Exporter;   
@ISA = ('Exporter');

@EXPORT=qw(rtmail maildomain) ;

use Mail::Sender ;
use MIME::Base64 ; 
use strict ; 

sub fixlist { 
    my ($s) = @_ ;
    if ( $s =~ / / ) { 
	$s =~ s/^\s+// ; 
	$s =~ s/s+$// ; 
	my @list = split(/[\s,]/, $s ) ; 
	my %m ;
	foreach (@list) { 
	    $m{$_} = 1 ; 
	}
	$s = join(',', sort keys %m) ; 
    }
    return $s ; 
}


sub rtmail { 
    my $retcode = 0 ; 
    my %hash = @_ ;
    my $to = &fixlist($hash{-to}) ; 
    my $from = $hash{-from} ; 
    my $subject = $hash{-subject} ; 
    my $cc = &fixlist($hash{-cc}) ; 
    my $bcc = &fixlist($hash{-bcc}) ; 
    my $attach = &fixlist($hash{-attach}) ; 
    my $msg = $hash{-msg} ; 

    if ( $hash{-background} ) { 
	my $pid ; 
	if ( $pid = fork ) {
	    # parent
	    return ; 
	} elsif ( defined $pid )   {
	    # child
	    close STDIN ; 
	    if ( $] > 5 ) {
		eval "setpgrp 0" ;
	    } else {
		setpgrp (0, $$) ;
	    }
	} else {
	    die "Can't fork\n" ;
	}
    }

    if ( ! defined $from || $from =~ /^\s*$/) { 
	$from = getpwuid($<);
    }

    my ($host, $domain) = &maildomain() ;
    if ( $from !~ /\@/ ) { 
	$from = "$from\@$domain" ; 
    }

# mail.brtt.com and its ip address are last gasp efforts for license requests and incident reports
    my @mailers = ($host, "localhost", "mailhost",  "mail", "mail.brtt.com", "207.174.76.131" ) ;
    my @tried = () ;
    my $result = -1 ;
    foreach my $mailer (@mailers) { 
	my ($mailhost) = gethostbyname($mailer) ; 
	if (defined $mailhost ) { 
	    $result = &send_the_mail( $from, $subject, $to, $cc, $bcc, $mailhost, $msg, $attach) ; 
	    push(@tried, $mailer) ;
	    if ( $result == 0 ) { 
		$msg = "sent mail using $mailhost to $to from $from\n" ; 
		last ;
	    }
	}
    }
    if ( $result != 0 ) { 
	$msg = "failed to send mail from $from using any of these servers: @mailers\n" ;
    }

    return ($result, $msg) ;
}

sub maildomain {
    chomp ( my $host = `uname -n` ) ;
    chomp ( my $os = `uname -s` ) ;
    my $domain ;
    if ( $os eq "Linux" ) {
        chomp($domain = `hostname --domain` );
    } else {
        open ( RESOLV, "/etc/resolv.conf" ) ;
        while ( <RESOLV> ) {
            if ( /^\s*domain\s+(\S+)/ ) {
                $domain = $1 ;
                last ;
            }
        }
        close RESOLV ;
    }

    if ( $host !~ /\w+\.\w+\.\w+/ ) {
        $host = "$host.$domain" ;
    }
    return ($host, $domain) ;
}

sub send_the_mail {
    my ($from, $subject, $to, $cc, $bcc, $mailhost, $msg, $attach) = @_ ;
    # print STDERR "from='$from'\n\t subject='$subject'\n\t to='$to'\n\t cc='$cc'\n\t bcc='$bcc'\n\t mailhost='$mailhost'\n\t msg='$msg'\n\t attach='$attach'\n\t" ;

    my $sender = new Mail::Sender
	{smtp => $mailhost, from => "$from"} ;
    $msg = " " if $msg eq "" ;
    my $result ;
    my $retcode = -1 ;
    if ( defined $attach && length($attach) > 0) { 
	$result = $sender->MailFile({
	    to => $to,
	    cc => $cc,
	    bcc => $bcc,
	    subject => $subject,
	    msg => $msg,
	    file => $attach,
	    }) ;
	if ( ! ref($result) && ( $result == -2 || $result == -3 || $result == -5 || $result == -7 ))  { 
	    warn ( "rtmail to $mailhost failure: Subject: $subject : $Mail::Sender::Error\n" ) ;
	} else { 
	    if ( ! ref($result) ) { 
		warn ( "rtmail to $mailhost failed: Subject: $subject : $Mail::Sender::Error\n" ) ;
	    } else {
		$retcode = 0 ; 
	    }
	}
    } else { 
	$result = $sender->MailMsg({
	    to => $to,
	    cc => $cc, 
	    bcc => $bcc,
	    subject => $subject,
	    msg => $msg,
	    }) ;
	if ( ! ref($result) && ( $result == -2 || $result == -3 || $result == -5 || $result == -7 ))  { 
	    warn ( "rtmail to $mailhost failure: Subject: $subject : $Mail::Sender::Error\n" ) ;
	} else { 
	    if ( ! ref($result) ) { 
		warn ( "rtmail to $mailhost failed: Subject: $subject : $Mail::Sender::Error\n" ) ;
	    } else {
		$retcode = 0 ; 
	    }
	}
    }
    $sender->Close ;
    return $retcode ; 
}

1;
